-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "Syn_SwitchIcons"

-- **************************************************
-- General information about this script
-- **************************************************

Syn_SwitchIcons = {}


Syn_SwitchIcons.recolor = 0
--                        ^ change this value to 1 for colorized icons


function Syn_SwitchIcons:Name()
	return "Switch Icons"
end

function Syn_SwitchIcons:Version()
	return "2.7"
end

function Syn_SwitchIcons:Description()
	return "Select switch by icons"
end

function Syn_SwitchIcons:Creator()
	return "(c)2024 J.Wesley Fowler (synthsin75), w/code contributed by Alexandra Evseeva and Lukas Krepel"
end

function Syn_SwitchIcons:UILabel()
	return "SYN: Switch Icons"
end

function Syn_SwitchIcons:LoadPrefs(prefs)
	self.size = prefs:GetInt("Syn_SwitchIcons.size", 64)
	self.shape = prefs:GetInt("Syn_SwitchIcons.shape", 0)
	self.sort = prefs:GetBool("Syn_SwitchIcons.sort", false)
	self.lSort = prefs:GetBool("Syn_SwitchIcons.lSort", false)
	self.h = prefs:GetInt("Syn_SwitchIcons.h", 0)
	self.w = prefs:GetInt("Syn_SwitchIcons.w", 0)
	self.pad = prefs:GetInt("Syn_SwitchIcons.pad", 8)
	self.speed = prefs:GetFloat("Syn_SwitchIcons.speed", 2)
end

function Syn_SwitchIcons:SavePrefs(prefs)
	prefs:SetInt("Syn_SwitchIcons.size", self.size)
	prefs:SetInt("Syn_SwitchIcons.shape", self.shape)
	prefs:SetBool("Syn_SwitchIcons.sort", self.sort)
	prefs:SetBool("Syn_SwitchIcons.lSort", self.lSort)
	prefs:SetInt("Syn_SwitchIcons.h", self.h)
	prefs:SetInt("Syn_SwitchIcons.w", self.w)
	prefs:SetInt("Syn_SwitchIcons.pad", self.pad)
	prefs:SetFloat("Syn_SwitchIcons.speed", self.speed)
end

function Syn_SwitchIcons:ResetPrefs()
	self.h = 0
	self.w = 0
end

-- **************************************************
-- Recurring values
-- **************************************************

Syn_SwitchIcons.ICON = {}
Syn_SwitchIcons.ID = -1
Syn_SwitchIcons.size = 64
Syn_SwitchIcons.shape = 0	--0=horizontal, 1=vertical, 2=square
Syn_SwitchIcons.sort = false
Syn_SwitchIcons.lSort = false
Syn_SwitchIcons.switchID = -1
Syn_SwitchIcons.lastLayer = false
Syn_SwitchIcons.cnt = 0
Syn_SwitchIcons.h = 0
Syn_SwitchIcons.w = 0
Syn_SwitchIcons.docFolder = nil
Syn_SwitchIcons.activate = nil
Syn_SwitchIcons.active = nil
Syn_SwitchIcons.pad = 8
Syn_SwitchIcons.speed = 2

Syn_SwitchIcons.min = .1
Syn_SwitchIcons.max = 40

-- **************************************************
-- Switch icon dialogs
-- **************************************************

local Syn_SwitchIconsDialog = {}

function Syn_SwitchIconsDialog:new(moho, layer)
	local d = LM.GUI.SimpleDialog(layer:Name(), Syn_SwitchIconsDialog)
	local l = d:GetLayout()
	
	d.moho = moho
	d.layer = layer
	d.frame = moho.frame
	d.switch = moho:LayerAsSwitch(layer)
	d.doc = moho.document
	
	if (layer:LayerType() == MOHO.LT_SWITCH) and (not moho:DrawingMesh()) then
		local frame = moho.frame
		local switch = moho:LayerAsSwitch(layer)
		local group = moho:LayerAsGroup(layer)
		local doc = string.match(moho.document:Name(), "(.+)%..+")
		local par = layer:UUID()
				
		l:PushH()
		
		local incrm
		local rep
		if (Syn_SwitchIcons.shape == 0) then
			--horizontal rows
			incrm = 15
			rep = incrm + 1
			if (Syn_SwitchIcons.w ~= 0) then
				incrm = math.ceil(Syn_SwitchIcons.w/Syn_SwitchIcons.size)-1
				rep = incrm + 1
			end
		elseif (Syn_SwitchIcons.shape == 2) then
			--arrange in square
			incrm = math.ceil(math.sqrt(group:CountLayers()))-1
			rep = incrm + 1
		else
			--vertical rows
			incrm = 9
			rep = incrm + 1
			if (Syn_SwitchIcons.h ~= 0) then
				incrm = math.ceil(Syn_SwitchIcons.h/Syn_SwitchIcons.size)-1
				rep = incrm + 1
			end
			l:PushV()
		end
		
		local a, b, c = 0, group:CountLayers()-1, 1
		if (Syn_SwitchIcons.sort) then
			a, b, c = group:CountLayers()-1, 0, -1
		end
		
		for i=a, b, c do
			local layer = group:Layer(i)
			local name = layer:Name()
			
			--which size icons?
			local size = "user"
			if (Syn_SwitchIcons.size == 64) then
				size = "lg"
			elseif (Syn_SwitchIcons.size == 36) then
				size = "sm"
			end
			
			local fileName = doc.."/"..par.."_"..layer:UUID().."_"..size
			
			local tempPath = moho:UserAppDir().."/scripts/ScriptResources/syn_resources/"..fileName..".png"
			--tempPath = string.gsub(tempPath, "\\", "/")
			
			moho:BeginFileListing(moho:UserAppDir().."/scripts/ScriptResources/syn_resources/"..doc.."/")
			local file = moho:GetNextFile()
			
			while (file) do
				if (string.sub(file, 1, -16) == par.."_"..layer:UUID().."_"..size) then
					tempPath = moho:UserAppDir().."/scripts/ScriptResources/syn_resources/"..doc.."/"..file
					break
				end
				file = moho:GetNextFile()
			end
			
			local f = io.open(tempPath, "rb")
			local image = true
			if (f == nil) then
				image = false
			else
				fileName = doc.."/"..file
				io.close(f)
			end
			local path = "ScriptResources/syn_resources/"..string.sub(fileName, 1, -5)
			
			Syn_SwitchIcons.ICON.i = MOHO.MSG_BASE + 30 + i
			
			if (not image) then
				self.icon = LM.GUI.Button(name, Syn_SwitchIcons.ICON.i)
			else
				self.icon = LM.GUI.ImageButton(path, name, false, Syn_SwitchIcons.ICON.i)
			end
			
			l:AddChild(self.icon, LM.GUI.ALIGN_LEFT)
			if (i/incrm == 1) then
				if (Syn_SwitchIcons.shape == 1) then
					--vertical rows
					incrm = incrm + rep
					l:Pop()
					l:PushV()
				else
					--horizontal rows
					--arrange in square
					incrm = incrm + rep
					l:Pop()
					l:PushH()
				end
			end
		end
		l:Pop()
	end
	return d
end

function Syn_SwitchIconsDialog:UpdateWidgets()
	
end

function Syn_SwitchIconsDialog:HandleMessage(msg)
	local ID = msg - (MOHO.MSG_BASE + 30)
	if (ID < 0) then return end
	Syn_SwitchIcons.ID = ID
	Syn_SwitchIcons.active = ID
	
	self.doc:PrepUndo(self.layer)
	self.doc:SetDirty()
	
	local layer = self.layer
	local val = self.switch:Layer(ID):Name()
	--print(val)
	--self.switch:SwitchValues().value:Set(val)
	--self.switch:SwitchValues():StoreValue()
--\/A.Evseeva
	local switchChannel = self.switch:Layer(ID):AncestorSwitchLayer():SwitchValues()
	switchChannel.value:Set(val)
	switchChannel:StoreValue()
--/\A.Evseeva
	--self.moho:UpdateUI()	--causes crash in v13.5.2
	MOHO.Redraw()
end


-- **************************************************
-- Settings dialog
-- **************************************************

local Syn_SwitchIconsSettingsDialog = {}
Syn_SwitchIconsSettingsDialog.SELECTITEM = MOHO.MSG_BASE + 12
Syn_SwitchIconsSettingsDialog.SELECTSHAPE = MOHO.MSG_BASE + 1000
--Syn_SwitchIconsSettingsDialog.DELETE = MOHO.MSG_BASE + 7
Syn_SwitchIconsSettingsDialog.SIZE = MOHO.MSG_BASE + 8
Syn_SwitchIconsSettingsDialog.SORT = MOHO.MSG_BASE + 9
Syn_SwitchIconsSettingsDialog.LSORT = MOHO.MSG_BASE + 10
Syn_SwitchIconsSettingsDialog.PAD = MOHO.MSG_BASE + 11

function Syn_SwitchIconsSettingsDialog:new(moho)
	local d = LM.GUI.SimpleDialog("", Syn_SwitchIconsSettingsDialog)
	d.moho = moho
	d.frame = moho.frame
	local l = d:GetLayout()
	
	--l:AddChild(LM.GUI.StaticText("-- Hit <enter> twice to apply changes --"))
	l:AddChild(LM.GUI.StaticText("-- Switch Icons Settings --"))
	--l:AddPadding()
	
	l:PushH()
		l:PushV()
		
		l:AddChild(LM.GUI.StaticText("Icon size:"), LM.GUI.ALIGN_LEFT)
		l:AddChild(LM.GUI.StaticText("Arrangement:"), LM.GUI.ALIGN_LEFT)
		l:AddChild(LM.GUI.StaticText("Reverse sort:"), LM.GUI.ALIGN_LEFT)
		l:AddChild(LM.GUI.StaticText("Spacing:"), LM.GUI.ALIGN_LEFT)
		
		l:Pop()
		l:PushV()
			l:PushH()
		
			d.menu = LM.GUI.Menu("Icon size")
			d.popup = LM.GUI.PopupMenu(75, true)
			d.popup:SetMenu(d.menu)
			l:AddChild(d.popup)
			d.childName = LM.GUI.DynamicText("", 0)
			l:AddChild(d.childName)
			
			l:AddPadding(-20)
			
			d.size = LM.GUI.TextControl(35, Syn_SwitchIcons.size, self.SIZE, LM.GUI.FIELD_UINT)
			d.size:SetWheelInc(1)
			l:AddChild(d.size)
			
			l:AddChild(LM.GUI.StaticText("px"))
			
			l:Pop()
			l:PushH()
			
			d.sMenu = LM.GUI.Menu("Arrange icons")
			d.shape = LM.GUI.PopupMenu(140, true)
			d.shape:SetMenu(d.sMenu)
			l:AddChild(d.shape)
			d.childName = LM.GUI.DynamicText("", 0)
			l:AddChild(d.childName)
			
			l:Pop()
			l:PushH()
			
			d.sort = LM.GUI.CheckBox("Icons", self.SORT)
			l:AddChild(d.sort)
			
			l:AddPadding()
			
			d.lSort = LM.GUI.CheckBox("Switches", self.LSORT)
			l:AddChild(d.lSort)
			
			l:Pop()
			l:PushH()
			
			d.padding = LM.GUI.TextControl(35, Syn_SwitchIcons.padding, self.PAD, LM.GUI.FIELD_UINT)
			d.padding:SetWheelInc(1)
			d.padding:SetConstantMessages(true)
			l:AddChild(d.padding)
			
			l:Pop()
		l:Pop()
	l:Pop()
	
	l:AddPadding()
	
	l:AddChild(LM.GUI.StaticText("Hit <enter> twice to apply"), LM.GUI.ALIGN_CENTER)
	
	--l:AddChild(LM.GUI.StaticText("-- Hit <enter> twice to apply changes --"))
	l:AddChild(LM.GUI.Divider(false), LM.GUI.ALIGN_FILL)
	
	l:PushH()
	l:AddChild(LM.GUI.StaticText("Workspace scrub time speed:    "), LM.GUI.ALIGN_LEFT)
	d.speedText = LM.GUI.TextControl(0, "00.00", Syn_SwitchIcons.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
	l:AddChild(d.speedText, LM.GUI.ALIGN_RIGHT)
	l:Pop()
	
	l:AddChild(LM.GUI.Button("Delete all thumbnails", --[[self]]Syn_SwitchIcons.DELETE), LM.GUI.ALIGN_LEFT)
	
	return d
end

function Syn_SwitchIconsSettingsDialog:UpdateWidgets()
	self.menu:RemoveAllItems()
	self.menu:AddItem("Large", 0, self.SELECTITEM)
	self.menu:AddItem("Small", 0, self.SELECTITEM + 1)
	self.menu:AddItem("Custom", 0, self.SELECTITEM + 2)
	
	self.sMenu:RemoveAllItems()
	self.sMenu:AddItem("Horizontal", 0, self.SELECTSHAPE)
	self.sMenu:AddItem("Vertical", 0, self.SELECTSHAPE + 1)
	self.sMenu:AddItem("Square", 0, self.SELECTSHAPE + 2)
	
	if (Syn_SwitchIcons.size == 64) then
		self.menu:SetCheckedLabel("Large", true)
		self.size:Enable(false)
		self.size:SetValue(64)
	elseif (Syn_SwitchIcons.size == 36) then
		self.menu:SetCheckedLabel("Small", true)
		self.size:Enable(false)
		self.size:SetValue(36)
	else
		self.menu:SetCheckedLabel("Custom", true)
		self.size:Enable(true)
		self.size:SetValue(Syn_SwitchIcons.size)
	end
	
	if (Syn_SwitchIcons.shape == 0) then
		self.sMenu:SetCheckedLabel("Horizontal", true)
	elseif (Syn_SwitchIcons.shape == 1) then
		self.sMenu:SetCheckedLabel("Vertical", true)
	else
		self.sMenu:SetCheckedLabel("Square", true)
	end
	
	self.sort:SetValue(Syn_SwitchIcons.sort)
	self.lSort:SetValue(Syn_SwitchIcons.lSort)
	self.padding:SetValue(Syn_SwitchIcons.pad)
	self.speedText:SetValue(Syn_SwitchIcons.speed)
	
	self.shape:Redraw()
	self.popup:Redraw()
end

function Syn_SwitchIconsSettingsDialog:OnOK()
	self:HandleMessage(Syn_SwitchIcons.DLOG_CHANGE) -- send this final message in case the user is in the middle of editing some value
end

function Syn_SwitchIconsSettingsDialog:HandleMessage(msg)
	if (msg == self.SELECTITEM + 2) then
		self.size:Enable(true)
		--Syn_SwitchIcons.size = self.size:IntValue()
		if (self:Validate(self.size, 36, 300)) then
			Syn_SwitchIcons.size = self.size:IntValue()
		else
			if (self.size:IntValue() < 36) then
				Syn_SwitchIcons.size = 36
			else
				Syn_SwitchIcons.size = 300
			end
		end
		self.size:SetValue(Syn_SwitchIcons.size)
	elseif  (msg == self.SELECTITEM + 1) then
		self.size:Enable(false)
		Syn_SwitchIcons.size = 36
		self.size:SetValue(36)
	elseif (msg == self.SELECTITEM) then
		self.size:Enable(false)
		Syn_SwitchIcons.size = 64
		self.size:SetValue(64)
	end
	
	if (msg == self.SELECTSHAPE + 2) then
		Syn_SwitchIcons.shape = 2
	elseif (msg == self.SELECTSHAPE + 1) then
		Syn_SwitchIcons.shape = 1
	elseif  (msg == self.SELECTSHAPE) then
		Syn_SwitchIcons.shape = 0
	end
	
	if (self:Validate(self.size, 36, 300)) then
		Syn_SwitchIcons.size = self.size:IntValue()
	else
		if (self.size:IntValue() < 36) then
			Syn_SwitchIcons.size = 36
		else
			Syn_SwitchIcons.size = 300
		end
	end
	
	if (msg == self.SORT) then
		Syn_SwitchIcons.sort = self.sort:Value()
	elseif (msg == self.LSORT) then
		Syn_SwitchIcons.lSort = self.lSort:Value()
	elseif (msg == self.PAD) then
		Syn_SwitchIcons.pad = LM.Clamp(self.padding:Value(), 0, 300)
		self.padding:SetValue(Syn_SwitchIcons.pad)
	end
	
	Syn_SwitchIcons.speed = LM.Clamp(self.speedText:FloatValue(), Syn_SwitchIcons.min, Syn_SwitchIcons.max)
	
	if (msg == --[[self]]Syn_SwitchIcons.DELETE) and (Syn_SwitchIcons.docFolder) then
		local button = LM.GUI.Alert(LM.GUI.ALERT_QUESTION, "Are you sure you want to delete all switch thumbnails for this project?", nil, nil, "Yes", "No", nil)
		if (button == 0) then
			os.execute("del /q /S ".."\""..Syn_SwitchIcons.docFolder.."\"")
			os.execute("rmdir ".."\""..Syn_SwitchIcons.docFolder.."\"")
			local removeDir = string.gsub(Syn_SwitchIcons.docFolder, " ", "\\ ")
			os.execute("rm -r "..removeDir) --OSX
		end
	end
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

Syn_SwitchIcons.DIA = {}
Syn_SwitchIcons.UPDATE = MOHO.MSG_BASE + 1
Syn_SwitchIcons.ALT_UPDATE = MOHO.MSG_BASE + 2
Syn_SwitchIcons.SELECTITEM = MOHO.MSG_BASE + 3
Syn_SwitchIcons.DLOG_BEGIN = MOHO.MSG_BASE + 4
Syn_SwitchIcons.DLOG_END = MOHO.MSG_BASE + 5
Syn_SwitchIcons.DLOG_CHANGE = MOHO.MSG_BASE + 6
Syn_SwitchIcons.DELETE = MOHO.MSG_BASE + 7

function Syn_SwitchIcons:DoLayout(moho, layout)
	--self.update = LM.GUI.Button("UD", self.UPDATE)
	--self.update:SetToolTip("Generate/update switch thumbnails (hold <alt> for whole project)")
	local path = "ScriptResources/syn_resources/syn_update"
	if (self.ver > 11) then
		self.update = LM.GUI.ImageButton(path, "Generate/update switch thumbnails (hold <alt> for whole project)", false, self.UPDATE, self.recolor)
	else
		self.update = LM.GUI.ImageButton(path, "Generate/update switch thumbnails (hold <alt> for whole project)", false, self.UPDATE)
	end
	self.update:SetAlternateMessage(self.ALT_UPDATE)
	
	layout:AddChild(self.update)
	if (self:OS() ~= "win") then
		layout:AddPadding(-16)
	end
	
	self.optDlog = Syn_SwitchIconsSettingsDialog:new(moho)
	
	self.opt = LM.GUI.PopupDialog("", --[[true]]false, self.DLOG_BEGIN)
	self.opt:SetDialog(self.optDlog)
	if (self.ver > 11) then
		self.opt:SetToolTip("Settings")
	end
	layout:PushV(LM.GUI.ALIGN_CENTER)
	layout:AddChild(self.opt, LM.GUI.ALIGN_CENTER, -15)
	layout:Pop()
		
	layout:AddChild(LM.GUI.Divider(true), LM.GUI.ALIGN_FILL)
	
	local group
	local recur = {}
	local doc = moho.document
	if (doc:CountSelectedLayers() > 1) then
		for i=0, doc:CountSelectedLayers()-1 do
			local layer = doc:GetSelectedLayer(i)
			if (layer:LayerType() == MOHO.LT_SWITCH) then
				local switch = moho:LayerAsSwitch(layer)
				if (not switch:IsFBFLayer()) then
					table.insert(recur, layer)
				end
			end
		end
	elseif (moho.layer:LayerType() == MOHO.LT_SWITCH) and (not moho:DrawingMesh()) then
		recur = {moho.layer}
	elseif (moho.layer:LayerType() == MOHO.LT_BONE) or (moho.layer:LayerType() == MOHO.LT_GROUP) then
	--DoLayout doesn't get called for a group layer when toggling to frame zero 
		group = moho:LayerAsGroup(moho.layer)
		recur = Syn_SwitchIcons:Recursive(moho, group, recur)
	else
		return
	end
	
	local a, b, c = 1, #recur, 1
	if (self.lSort) then
		a, b, c = #recur, 1, -1
	end
	
	for i=a, b, c do
		local layer = recur[i]
		if (layer:LayerType() == MOHO.LT_SWITCH) and (not moho:DrawingMesh()) then
		
			self.iconDlog = Syn_SwitchIconsDialog:new(moho, layer)
			
			self.DIA.i = MOHO.MSG_BASE + 7 + i
			
			self.popup = LM.GUI.PopupDialog(layer:Name(), true--[[false]], self.DIA.i)	--hold open
			self.popup:SetDialog(self.iconDlog)
			if (self.ver > 11) then
				self.popup:SetToolTip("Hit <enter> to update UI")
			end
			layout:AddChild(self.popup)
			
			layout:AddPadding(Syn_SwitchIcons.pad)
		end
	end
	self.recur = #recur
	
--[[--this cannot tell the difference from switching tabs between two open documents
	if (self.proj) and (self.proj ~= moho.document:Name()) then --document saved as new name
		local opSys = self:OS()
		local last = string.match(self.proj, "(.+)%..+")
		local doc = string.match(moho.document:Name(), "(.+)%..+")
		local appDir = string.gsub(moho:UserAppDir(), "\\", "/")
		local old = appDir.."/scripts/ScriptResources/syn_resources/"..last
		local new = appDir.."/scripts/ScriptResources/syn_resources/"..doc
		if (opSys ~= "win") then
			old = string.sub(old, " ", "\\ ")
			new = string.sub(new, " ", "\\ ")
			--os.execute("mkdir "..new)
			os.execute("mv "..old.." "..new)
		end
		os.rename(old, new) --rename thumbnail folder
	end
--]]
	self.proj = moho.document:Name()
end

function Syn_SwitchIcons:UpdateWidgets(moho)
	if (moho.frame == 0) --[[and (moho.layer:LayerType() == MOHO.LT_SWITCH) and (not moho:DrawingMesh())]] then
		self.update:Enable(true)
	else
		self.update:Enable(false)
	end
end 

function Syn_SwitchIcons:HandleMessage(moho, view, msg)
	if (msg == self.UPDATE) then
		if (moho:UserAppDir() == "") then
			LM.GUI.Alert(LM.GUI.ALERT_INFO, "You must have a Custom Content Folder to store switch thumbnails.", nil, nil, "OK", nil, nil)
		else
			local count = 0
			local layer = moho.document:LayerByAbsoluteID(count)
			repeat
				if (layer:SecondarySelection()) or (self.ver < 12 and layer == moho.layuer) then
					self:UpdateIcons(moho, view, layer)
				end
				count = count+1
				layer = moho.document:LayerByAbsoluteID(count)
			until (not layer)
		end
	elseif (msg == self.ALT_UPDATE) then
		if (moho:UserAppDir() == "") then
			LM.GUI.Alert(LM.GUI.ALERT_INFO, "You must have a Custom Content Folder to store switch thumbnails.", nil, nil, "OK", nil, nil)
		else
			local count = 0
			local layer = moho.document:LayerByAbsoluteID(count)
			repeat
				self:UpdateIcons(moho, view, layer)
				count = count+1
				layer = moho.document:LayerByAbsoluteID(count)
			until (not layer)
		end
	end
	if (--[[msg == self.DLOG_END or ]]msg == self.UPDATE or msg == self.ALT_UPDATE or msg == self.DELETE) then
		local frame = moho.frame
		if (frame == 0) then
			moho:SetCurFrame(1)
		else
			moho:SetCurFrame(0)
		end
		moho:SetCurFrame(frame)
	end
	
	local ID = msg - (MOHO.MSG_BASE + 7)
	self.switchID = ID
	
	if (msg == self.DLOG_BEGIN) then
		self.optDlog.doc = moho.document
	end
	
	self.h = view:Graphics():Height()
	self.w = view:Graphics():Width()
	
	--MOHO.Redraw()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function Syn_SwitchIcons:UpdateIcons(moho, view, layer)
	if (layer:LayerType() == MOHO.LT_SWITCH) and (not moho:DrawingMesh()) then
		local group = moho:LayerAsGroup(layer)
		local doc = string.match(moho.document:Name(), "(.+)%..+")
		local par = layer:UUID()
		local H,W = moho.document:Height(), moho.document:Width()
		local switch = moho:LayerAsSwitch(layer)
		local swVal = switch:GetValue(0)
				
		local opSys = self:OS()
		
		--[[local baseDir = "\""..moho:UserAppDir().."\""
		
		local dir = baseDir.."/scripts/ScriptResources/syn_resources/".."\""..doc.."\""
		os.execute("mkdir "..dir)]]
		
-- **************************************************
-- Lukas Krepel @ http://www.lukaskrepel.nl/
-- http://www.lostmarble.com/forum/viewtopic.php?p=196543#p196543
-- **************************************************
		local dir
		if opSys == "win" then
			dir = "\""..moho:UserAppDir().."\"".."/scripts/ScriptResources/syn_resources/".."\""..doc.."\""
		else
			dir = moho:UserAppDir().."/scripts/ScriptResources/syn_resources/"
			dir = string.gsub(dir, " ", "\\ ")
			os.execute("mkdir "..dir)
			doc = string.gsub(doc, " ", "\\ ")
			dir = dir..doc
		end
		os.execute("mkdir "..dir)
-- **************************************************
		
		local size = "user"
		if (self.size == 64) then
			size = "lg"
		elseif (self.size == 36) then
			size = "sm"
		end
		
		--delete old icons
		moho:BeginFileListing(moho:UserAppDir().."/scripts/ScriptResources/syn_resources/"..doc.."/")
		local file = moho:GetNextFile()
		local subpar = string.gsub(par, "%-", "%%-")
		while (file) do
			for i=0, group:CountLayers()-1 do
				local l = switch:Layer(i)
				local uid = string.gsub(l:UUID(), "%-", "%%-")
				--[[if (string.match(file, "("..subpar.."_"..uid.."_"..size.."_"..".+"..")")) then
					--os.execute("del /q /S ".."\""..moho:UserAppDir().."/scripts/ScriptResources/syn_resources/"..doc.."/"..file.."\"")
					os.remove(moho:UserAppDir().."/scripts/ScriptResources/syn_resources/"..doc.."/"..file)
					break
				end]]
				
-- **************************************************
-- Lukas Krepel @ http://www.lukaskrepel.nl/
-- http://www.lostmarble.com/forum/viewtopic.php?p=196543#p196543
-- **************************************************
				if (string.match(file, "("..subpar.."_"..uid.."_"..size.."_"..".+"..")")) then
					local filepath = moho:UserAppDir().."/scripts/ScriptResources/syn_resources/"..doc.."/"..file
					if opSys == "win" then
						os.remove(filepath)
					else
						filepath = string.gsub(filepath, " ", "\\ ")
						os.execute ("rm "..filepath)
					end
					break
				end
-- **************************************************

			end
			file = moho:GetNextFile()
		end
		
		for i=0, group:CountLayers()-1 do
			local layer = group:Layer(i)
			local name = layer:Name()
			switch:SetValue(0, name)
			
-- **************************************************
-- Alexandra Evseeva @ http://ae.revival.ru/
-- http://www.lostmarble.com/forum/viewtopic.php?p=196232#p196232
-- **************************************************
			--get global layer matrix and transform layer bounds with it to make them global
			local layerGlobal = self:GetGlobalLayerMatrix(moho, layer, moho.frame)
			local BB = layer:Bounds(moho.frame)
			layerGlobal:Transform(BB)
			--place camera to zoom to global layer bounds 
			local oldCameraPos = moho.document.fCameraTrack:GetValue(moho.frame)
			local newCameraPos = LM.Vector3:new_local()
			newCameraPos:Set(BB:Center2D().x, BB:Center2D().y, oldCameraPos.z)
			local oldZoom = moho.document.fCameraZoom:GetValue(moho.frame)
			moho.document:SetShape(self.size, self.size)
			moho.document.fCameraTrack:SetValue(moho.frame, newCameraPos)
			local camAngle = 2.0 * math.atan(BB:MaxDimension2D()/(newCameraPos.z * 2))
			local newZoom = 60 / (180.0 * camAngle/math.pi)
			moho.document.fCameraZoom:SetValue(moho.frame, newZoom)
-- **************************************************
						
			local rand = self:RandomName()
			local fileName = par.."_"..layer:UUID().."_"..size.."_"..rand
			
			local path = moho:UserAppDir().."/scripts/ScriptResources/syn_resources/"..doc.."/"..fileName..".png"
			
			moho:FileRender(path)
			moho.document.fCameraZoom:SetValue(moho.frame, oldZoom)
			moho.document.fCameraTrack:SetValue(moho.frame, oldCameraPos)
		end
		
		switch:SetValue(0, swVal)
		moho.document:SetShape(W,H)
		
		MOHO.Redraw()
	end
end

-- **************************************************
-- Standard functions
-- **************************************************

function Syn_SwitchIcons:ColorizeIcon()
	if (self.recolor == 1) then
		return true
	end
	return false
end

function Syn_SwitchIcons:NonDragMouseMove()
	return true
end

function Syn_SwitchIcons:IsEnabled(moho)
	if (moho.layer:LayerType() ~= MOHO.LT_BONE) and (moho.layer:LayerType() ~= MOHO.LT_GROUP) and (moho.layer:LayerType() ~= MOHO.LT_SWITCH) then
		--return false
	end
	return true
end

function Syn_SwitchIcons:IsRelevant(moho)
	local docName = string.match(moho.document:Name(), ".-([^\\/]-)%.?[^%.\\/]*$")
	self.docFolder = string.gsub(moho:UserAppDir().."/scripts/ScriptResources/syn_resources/"..docName.."/", "\\", "/")
	self.ver = tonumber(string.match(moho:AppVersion(), "(%d+)%p.+"))
	return true
end

function Syn_SwitchIcons:OnMouseDown(moho, mouseEvent)
	if (self.ver >= 13) then
		local app = string.match(moho:AppDir(), "(.+)Support")
		local cursor = LM.GUI.Cursor(app.."Images/curs_hresize")
		mouseEvent.view:SetCursor(cursor)
	end
	mouseEvent.view:DrawMe()
	self.startFrm = moho.frame
end

function Syn_SwitchIcons:OnMouseMoved(moho, mouseEvent)
	if (not mouseEvent.view:IsMouseDragging(0)) then
		if (self.ver >= 13) then
			local app = string.match(moho:AppDir(), "(.+)Support")
			local cursor = LM.GUI.Cursor(app.."Images/dragCursor")
			mouseEvent.view:SetCursor(cursor)
		end
		return
	else
		local range = math.ceil(((mouseEvent.pt.x - mouseEvent.startPt.x)/8)*self.speed)
		if (self.startFrm+range < 0) then
			moho:SetCurFrame(0)
		else
			moho:SetCurFrame(self.startFrm+range)
		end
	end
end

function Syn_SwitchIcons:OnMouseUp(moho, mouseEvent)
	mouseEvent.view:SetCursor()
	mouseEvent.view:DrawMe()
	moho:UpdateUI()	--needed to update timeline and layers window after switch is keyed
end

function Syn_SwitchIcons:OnKeyDown(moho, keyEvent)
--refresh settings/dialogs with ENTER
	local frame = moho.frame
	--if (keyEvent.keyCode == LM.GUI.KEY_RETURN) then --doesn't work with enter key
	if (keyEvent.keyCode == 13) then
		if (frame == 0) then
			moho:SetCurFrame(1)
		else
			moho:SetCurFrame(0)
		end
		moho:SetCurFrame(frame)
		
		keyEvent.view:DrawMe()
	end
end

-- **************************************************
-- Utility functions
-- **************************************************

function Syn_SwitchIcons:OS()
	local opSys
	if (os.getenv("OS") ~= nil) then
		opSys = string.lower(string.sub(os.getenv("OS"), 1, 3))
		if opSys == "win" then
			opSys = "win"
		else
			opSys = "unix"
		end
	else
		opSys = "unix"
	end
	return opSys
end

function Syn_SwitchIcons:RandomName()
	--random alphanumeric string
	local charset = {}
	for c = 48, 57  do table.insert(charset, string.char(c)) end
	for c = 65, 90  do table.insert(charset, string.char(c)) end
	for c = 97, 122 do table.insert(charset, string.char(c)) end
	length = 10
	local function randomString(length)
		if not length or length <= 0 then return '' end
		math.randomseed(math.floor(os.clock()^5))
		return randomString(length - 1) .. charset[math.random(1, #charset)]
	end
	return randomString(length)
end

function Syn_SwitchIcons:Recursive(moho, group, layers)
	local stack = {}
	local sp = 0
	for i=0, group:CountLayers()-1 do
		local layer = group:Layer(i)
		table.insert(layers, layer)
		local group = nil
		local layerID = 0
		while true do
			if (layer:IsGroupType()) then
				table.insert(stack, {group, layerID -1})
				sp = sp+1
				group = moho:LayerAsGroup(layer)
				layerID = group:CountLayers()
			end
			if (layerID > 0) then
				layerID = layerID -1
				layer = group:Layer--[[ByDepth]](layerID)
				table.insert(layers, layer)
			else
				layerID = -1
				while (sp > 0) do
					group, layerID = stack[sp][1], stack[sp][2]
					table.remove(stack)
					sp = sp -1
					if (layerID >= 0) then
						layer = group:Layer--[[ByDepth]](layerID)
						table.insert(layers, layer)
						break
					end
				end
			end
			if (layerID < 0) then
				break
			end
		end
	end
	return layers
end

-- **************************************************
-- Alexandra Evseeva @ http://ae.revival.ru/
-- http://www.lostmarble.com/forum/viewtopic.php?p=196232#p196232
-- http://mohoscripts.com/script/ae_utilities
-- **************************************************

function Syn_SwitchIcons:GetGlobalLayerMatrix(moho, layer, frame) 
	local prevMatrix = LM.Matrix:new_local()
	prevMatrix:Identity()
	local nextLayer = layer
	repeat
		local prevLayer = nextLayer
		local matrix = LM.Matrix:new_local()
		nextLayer:GetLayerTransform(frame, matrix, moho.document)
		matrix:Multiply(prevMatrix)
		prevMatrix:Set(matrix)
		if nextLayer:Parent() then nextLayer = nextLayer:Parent() end
	until nextLayer == prevLayer
	local cameraMatrix = LM.Matrix:new_local()
	moho.document:GetCameraMatrix(frame, cameraMatrix)
	cameraMatrix:Invert()
	cameraMatrix:Multiply(prevMatrix)
	return cameraMatrix
end

